#!/usr/bin/env python3
# coding: utf8

import functools
import logging


def compose(*functions):
    return functools.reduce(lambda f, g: lambda x: f(g(x)), functions, lambda x: x)


def execute_make_method(method_name, **params):
    s = 'API.{method_name}({params})'
    params = str(params).replace(" ", "").replace("'", "\"")
    return s.format(method_name=method_name, params=params)


def execute_make_code(methods):
    return 'return [{}];'.format(','.join(methods))


def offsets_iterator(offset, total, batch_size):
    res = []
    for i in range(offset, total+offset, offset):
        res.append(i)
        if len(res) == batch_size:
            yield res
            res = []
    yield res


def get_logger(name):

    level = logging.INFO

    fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    formatter = logging.Formatter(fmt)

    ch = logging.StreamHandler()
    ch.setLevel(level)
    ch.setFormatter(formatter)

    logger = logging.getLogger(name)
    logger.setLevel(level)
    logger.addHandler(ch)

    return logger



#!/usr/bin/env python3
# coding: utf8

from functools import partial
import re
from urllib.parse import urlparse, parse_qsl

import requests


class VK(object):

    def __init__(
            self,
            username=None,
            password=None,
            client_id=None,
            scope=None,
            version='5.69',
            access_token=None,
    ):

        default_scope = [
            'friends',
            'photos',
            'audio',
            'video',
            'pages',
            'status',
            'notes',
            'messages',
            'wall',
            'offline',
            'docs',
            'groups',
            'notifications',
            'stats',
            'email',
            'market'
        ]

        self._username = username
        self._password = password,
        self._client_id = client_id
        self._scope = scope or ','.join(default_scope)
        self._version = version
        self._access_token = access_token

        self._api_object_name = None

    def __getattr__(self, method_name):

        if not self._api_object_name:
            self._api_object_name = method_name
            return self

        method_name = '{}.{}'.format(self._api_object_name, method_name)
        self._api_object_name = None

        return partial(self._call_method, method_name)

    def __call__(self, method_name, **params):
        return getattr(self, method_name)(**params)

    def _get_auth_session(self):

        auth_url = 'https://login.vk.com/?act=login'
        re_ip_h = re.compile(r'name="ip_h" value="([a-z0-9]+)"')
        re_lg_h = re.compile(r'name="lg_h" value="([a-z0-9]+)"')

        session = requests.Session()

        html = session.get(auth_url).text
        ip_h = re.findall(re_ip_h, html)
        lg_h = re.findall(re_lg_h, html)

        data = {
            'act': 'login',
            'role': 'al_frame',
            '_origin': 'https://vk.com',
            'ip_h': ip_h,
            'lg_h': lg_h,
            'email': self._username,
            'pass': self._password
        }
        session.post(auth_url, data)
        return session

    def _get_access_token(self):

        if (not self._access_token
            and self._username
            and self._password
        ):
            session = self._get_auth_session()

            auth_api_url = 'https://oauth.vk.com/authorize'
            re_token_url = re.compile(r'https://login.vk.com/[?]act=grant_access.*?&https=1')

            params = {
                'client_id': self._client_id,
                'display': 'page',
                'redirect_uri': 'https://oauth.vk.com/blank.html',
                'scope': self._scope,
                'response_type': 'token',
                'v': self._version
            }

            response = session.get(auth_api_url, params=params)

            if 'access_token' not in response.url:
                html = response.text
                url = re.findall(re_token_url, html)[0]
                response = session.get(url)

            query = urlparse(response.url).fragment
            self._access_token = dict(parse_qsl(query))['access_token']

        return self._access_token

    def _call_method(self, method_name, **params):

        api_url = 'https://api.vk.com/method/{}'.format(method_name)

        access_token = self._get_access_token()
        if access_token:
            params['access_token'] = access_token

        params['v'] = self._version

        data = requests.post(api_url, params).json()
        return data

    def execute(self, **params):
        return self._call_method('execute', **params)



#!/usr/bin/env python3
# coding: utf8

from vk7.utils import (
    execute_make_method,
    execute_make_code,
    offsets_iterator,
    get_logger
)


logger = get_logger(__name__)


def objects_iterator(vk, method, params, total, verbose=False):

    for offsets in offsets_iterator(100, total, 25):

        if verbose:
            logger.info('{} {}'.format(method, offsets[-1]))

        methods = [
            execute_make_method(method, offset=offset, count=100, **params)
            for offset in offsets
        ]

        code = execute_make_code(methods)
        data = vk.execute(code=code)

        for batch in data['response']:
            for item in batch['items']:
                yield item


def group_members_iterator(vk, group_id, fields='sex', verbose=False):

    method = 'groups.getMembers'

    params = {
        'group_id': group_id,
        'sort': 'id_asc',
        'fields': fields
    }

    data = vk.groups.getMembers(offset=0, count=0, **params)
    total_members = data['response']['count']

    return objects_iterator(vk, method, params, total_members, verbose)


def docs_iterator(vk, q, verbose=False):

    method = 'docs.search'

    params = {
        'q': q
    }

    return objects_iterator(vk, method, params, 1000, verbose)



#!/usr/bin/env python3
# coding: utf8

from setuptools import setup, find_packages
from codecs import open
from os import path


NAME = 'vk7'
VERSION = __import__(NAME).__version__
DESCRIPTION = 'Framework for extract data from vk.com (russian social network).'
URL = 'https://github.com/pash4paul/vk7'
AUTHOR = 'Pavel Fomin'
AUTHOR_EMAIL = 'pash4paul@gmail.com'


cwd = path.abspath(path.dirname(__file__))
with open(path.join(cwd, 'README.rst'), encoding='utf-8') as f:
    LONG_DESCRIPTION = f.read()


setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    url=URL,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    keywords='api data osint social vk.com',
    packages=find_packages(exclude=['tests']),
    install_requires=['requests==2.18.4']
)
